package net.sf.corn.gate;

import java.io.Serializable;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;


/**
 * This class represents a request to call a service operation. A service request contains the information below:
 * <ol>
 * 	<li><b>Service Name:</b>Name of the target service to be called.This attribute is mandatory.</li>
 *  <li><b>Operation Name:</b>Target operation  to be executed.Mandatory</li>
 *  <li><b>Parameters:</b>Operation Parameters.Mandatory!</li>
 *  <li><b>Header:</b>Additional values to pass to services. Optional</li>
 *  <li><b>Client Context:</b>The context information that the client is actually bounded in current JVM
 *    .This is an optional attribue.Null value represents default client context which specified by the current platform.</li>
 * </ol>
 * @author Serhat Dirik
 *
 */
public class ServiceRequest implements Serializable{

	private static final long serialVersionUID = -8155977101098164364L;
	private String serviceName;
	private String operationName;
	private Object[] parameters;
	private Map<String,Object> headers = new ConcurrentHashMap<String, Object>();
	private IClientContext clientContext;
	
	public ServiceRequest(String serviceName, String operationName) {
 
		this(serviceName,operationName,new Object[]{});
	}
	
	public ServiceRequest(String serviceName, String operationName,
			Object[] parameters) {
 
		this(serviceName,operationName,parameters,null);
	}
	
	

	public ServiceRequest(String serviceName, String operationName,
			Object[] parameters, IClientContext clientContext) {
		this(serviceName,null,operationName,parameters,clientContext);
	}
	public ServiceRequest(String serviceName,String version,String operationName,
			Object[] parameters, IClientContext clientContext) {
		
		if(serviceName == null || serviceName.trim().length() == 0){
			throw new IllegalArgumentException("Null or empty Service Name!");
		}
		
		if(operationName == null || operationName.trim().length() == 0){
			throw new IllegalArgumentException("Null or empty Operation Name!");
		}
		this.serviceName = serviceName;
		this.operationName = operationName;
		this.parameters = parameters;
		this.clientContext = clientContext;
		
	}
	
	public String getServiceName() {
		return serviceName;
	}

	public String getOperationName() {
		return operationName;
	}

	public Object[] getParameters() {
		return parameters;
	}

	public IClientContext getClientContext() {
		return clientContext;
	}
	
	public boolean containsHeaderKey(String key) {
		return headers.containsKey(key);
	}

	public Object getFromHeader(String key) {
		return headers.get(key);
	}

	public Object addToHeader(String key, Object value) {
		return headers.put(key, value);
	}

	public Object removeFromHeader(String key) {
		return headers.remove(key);
	}

	public Set<String> headerKeySet() {
		return headers.keySet();
	}



	@Override
	public String toString() {
		return new StringBuilder().append("Service Request [serviceName:").append(serviceName).
		append(", operationName:").append(operationName).
		append(", parameters:").append(parameters).append(", headers:").append(headers).append(", clientContext:").
		append(clientContext).toString();
	}
	
	 
}
