package net.sf.corn.gate.jsonrpc;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ParsingException;
import net.sf.corn.converter.json.JsTypeComplex;
import net.sf.corn.converter.json.JsTypeObject;
import net.sf.corn.converter.json.JsTypeSimple;
import net.sf.corn.converter.json.JsonConverter;
import net.sf.corn.converter.json.JsonStringParser;


/**
 * &nbsp;&nbsp;This class represents a response from a remote json-rpc service. The
 * response of a service call may contain a result or an error. In any cases
 * call responses contains a message identifier which is the same as incoming
 * message for message correlation on the caller side. The response may contain
 * a header message part which contains additional fields beside error or
 * result. The following Javascript class represents a response wrapper which
 * contains an operation result within it.<br>
 * <code>
 *  {<br>
 *   &nbsp;"id":"64f4f4b0-79ee-45f0-9b47-77836bc74ce8",<br>
 *   &nbsp;"header":{ <br>
 *     &nbsp;&nbsp;"headerField1": "headerVal",<br>
 *     &nbsp;&nbsp;"headerField2": "headerVal",<br>
 *   &nbsp;},<br>
 *   &nbsp;"result":"resultObject"<br>
 *  }<br>
 * </code> 
 * 
 * <br>
 * 
 * The following Javascript class represents a response which contains an error within it.<br>
 * <code>
 *   {<br>
 *   &nbsp;"id":"C3F236B3-D450-0001-B8E3-4F8D528C5390",<br>
 *   &nbsp;"error":<br>
 *   	&nbsp;&nbsp;{"errorName":"org.sr.server.AuhenticationException",<br>
 *   	 &nbsp;&nbsp;"errorCode":491,<br>
 *       &nbsp;&nbsp;"errorMsg":"Authentication is required for  securityTestBean.authenticationRequiredMethod calls"<br>
 *       &nbsp;&nbsp;}<br>
 *    }
 * </code>
 * <hr>
 * <h2>Message Parts</h2>
 * <h3>id</h3>
 * &nbsp;This part should contain a unique message identifier for the message
 * correlation. Results of operations will be sent to the clients by using same
 * identifier as requests. This would help clients to correlate their responses
 * from server.
 * <h3>header</h3>
 * &nbsp; The header part of the message contains additional fields that sent by
 * the server beside parameters.
 * <h3>result</h3>
 * &nbsp;The result of the operation. In the case of an exception raises, the
 * wrapper class wouldn't contains result.
 * <h3>error</h3>
 * &nbsp;The wrapper class contains error part only if an exception raises from
 * the service. Error object has three attributes as errorCode,errorMsg and
 * errorName.
 * 
 * @author Serhat Dirik
 * 
 */
public class JsonRpcResponse implements Serializable {

	private static final long serialVersionUID = -6349761749792602787L;
	public static final String RESPONSEID_ATTRIBUTE = "id";
	public static final String HEADER_ATTRIBUTE = "header";
	public static final String RESULT_ATTRIBUTE = "result";
	public static final String ERROR_ATTRIBUTE = "error";
	public static final String ERRORNAME_ATTRIBUTE = "errorName";
	public static final String ERRORMESSAGE_ATTRIBUTE = "errorMsg";

	private JsTypeComplex enclosedResponse = new JsTypeComplex();
	private JsTypeObject result = null;
	private String errorMsg = null;
	private String errorName = null;
	private String id = null;
	private Map<String, Object> header = new HashMap<String, Object>();

	
	@SuppressWarnings("unused")
	private JsonRpcResponse() {
	}
	
	/**
	 * Constructs a new instance from a json string 
	 * @param jsonString a json syntaxed string that contains required fields
	 * @throws ConversionException
	 * @throws ParsingException
	 */
	JsonRpcResponse(String jsonString) throws ConversionException, ParsingException{
		JsTypeComplex json_resp = (JsTypeComplex) JsonStringParser.parseJsonString(jsonString);
		setResponse(json_resp);
	}
	
	/**
	 * Create a JSONResponse wrapper java class to the given complex json object
	 * 
	 * @param response complex typed json object
	 * @throws ConversionException
	 */
	JsonRpcResponse(JsTypeComplex response) throws ConversionException {
		setResponse(response);

	}
	
	/**
	 * Creates a response wrapper class for the given result for the given
	 * message id
	 * 
	 * @param id
	 * @param result
	 * @throws ConversionNotSupportedException 
	 * @throws ConversionException 
	 * @throws JSONException
	 */
	JsonRpcResponse(String id, JsTypeObject result) 
			throws ConversionException {
		enclosedResponse.put(RESPONSEID_ATTRIBUTE, new JsTypeSimple(JsTypeObject.quote(id),null));
		enclosedResponse.put(RESULT_ATTRIBUTE, result);
		this.id = id;
		this.result = result;
	}

	/**
	 * Creates a response wrapper class which contains the given error
	 * 
	 * @param id
	 * @param errorCode
	 * @param errorName
	 * @param errorMessage
	 * @throws ConversionNotSupportedException 
	 * @throws ConversionException 
	 * @throws JSONException
	 */
	JsonRpcResponse(String id,String errorName,
			String errorMessage) {
		try {
			enclosedResponse.put(RESPONSEID_ATTRIBUTE, new JsTypeSimple(JsTypeObject.quote(id),null));
			JsTypeComplex error = new JsTypeComplex();
			error.putObject(ERRORMESSAGE_ATTRIBUTE,  new JsTypeSimple(JsTypeObject.quote(errorMessage),null));
			error.putObject(ERRORNAME_ATTRIBUTE,  new JsTypeSimple(JsTypeObject.quote(errorName),null));
			enclosedResponse.put(ERROR_ATTRIBUTE, error);
		} catch (ConversionException e) {
			e.printStackTrace();
		}
		
		this.id = id;
		this.errorMsg = errorMessage;
		this.errorName = errorName;
	}

	
	private void setResponse(JsTypeComplex response) throws ConversionException{
		JsTypeObject jsObj = response.get(RESPONSEID_ATTRIBUTE);
		id = jsObj != null ? jsObj.toStringValue() : null;
		result = response.get(RESULT_ATTRIBUTE);
		errorMsg = null;
		errorName = null;
		if(result == null){
			JsTypeComplex error = (JsTypeComplex) response.get(ERROR_ATTRIBUTE);
			if(error != null){
				JsTypeObject jserr =   error.get(ERRORMESSAGE_ATTRIBUTE);
				errorMsg  =  jserr == null ? null : jserr.toStringValue();
				jserr =   error.get(ERRORNAME_ATTRIBUTE);
				errorName =  jserr == null ? null : jserr.toStringValue();
			}
		}
		
		header = new HashMap<String, Object>(0);
		JsTypeComplex headerJSON = (JsTypeComplex) response.get(HEADER_ATTRIBUTE);
		if(headerJSON != null){
			JsonConverter converter = new JsonConverter();
			for (String key : headerJSON.keys()) {
				JsTypeObject headerItem = headerJSON.get(key);
				Object headerObject = null;
				if (headerItem != null) {
					headerObject =  converter.toJava(headerItem);
				}
				header.put(key, headerObject);
			}
		}
		enclosedResponse = response;
	}
	
	public boolean hasResult() {
		return result != null;
	}

	public boolean hasHeaders() {
		return header != null && header.size() > 0;
	}

	public boolean hasError() {
		return errorName != null;
	}

	public String getResponseId() {
		return id;
	}

	public String getErrorName() {
		return errorName;
	}

	public String getErrorMessage() {
		return errorMsg;
	}

	public JsTypeObject getResult() {
		return result;
	}

	public Object getResultAsJava() throws ConversionException {
		JsonConverter converter = new JsonConverter();
		return converter.toJava(result);
	}
	
	void setHeader(Map<String, Object> header) throws
			ConversionException {
		this.header = header;
		enclosedResponse.remove(HEADER_ATTRIBUTE);
		if (header == null)
			return;
		JsTypeComplex jsHdr = new JsTypeComplex();
		for (String key : header.keySet()) {
			Object item = header.get(key);
			jsHdr.putObject(key, item);
		}
		enclosedResponse.put(HEADER_ATTRIBUTE, jsHdr);
	}

	public Map<String, Object> getHeader() {
		if (header == null)
			header = new HashMap<String, Object>();
		return Collections.unmodifiableMap(header);
	}

	void putToHeader(String fieldName, Object value)
			throws ConversionException  {
		if (header == null)
			header = new HashMap<String, Object>();
		header.put(fieldName, value);
		setHeader(header);
	}

	void removeFromHeader(String fieldName) throws 
			ConversionException {
		if (header == null)
			return;
		header.remove(fieldName);
		setHeader(header);
	}

	public JsTypeComplex toJSON() {
		return enclosedResponse;
	}

	@Override
	public String toString() {
		return enclosedResponse == null ? null : enclosedResponse.toString();
	}

}
