package net.sf.corn.gate.locators;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Hashtable;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import net.sf.corn.exception.ExceptionBuilder;
import net.sf.corn.exception.ExceptionCommand;
import net.sf.corn.gate.GateException;
import net.sf.corn.gate.ILocator;
import net.sf.corn.gate.ServiceDefinition;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This is component locator implementation class for Enterprise Java Beans that
 * implemented using EJB 2.1 and former specs. This locator access EJB instances
 * by using JNDI, so the target attribute of components need to show the JNDI
 * name of the target EJB. In addition to that this locator needs that the
 * component definitions describes the home interface class of the target EJB as
 * custom property.
 * 
 * <h2>Sample Configuration</h2> &lt;Config&gt;<br>
 * &nbsp;&lt;Locators&gt;<br>
 * &nbsp;&nbsp;&lt;Locator class="net.sf.corn.gate.locators.EJB2Locator" /&gt;<br>
 * &nbsp;&lt;/Locators&gt;<br>
 * &nbsp;&lt;Services&gt;<br>
 * &nbsp;&nbsp;&lt;Service name="echoBean" componentTarget="test.bean.EchoBean"
 * scope="singleton" locator="EJB2Locator"&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;Property key="homeInterfaceClass"
 * value="test.bean.EchoBeanHome"/&gt;<br>
 * &nbsp;&nbsp;&lt;/Service&gt;<br>
 * &nbsp;&lt;Services&gt;<br>
 * &lt;/Config&gt;<br>
 * <p>
 * It's also possible to set additional attributes for JNDI context
 * initialization such as context factory or jndi provider url. Additional
 * custom attributes are specified by {@link javax.naming.Context} classes
 * constants. For example in order to set {@link javax.naming.Context}
 * INITIAL_CONTEXT_FACTORY attribute, use contants string value as key which is
 * "java.naming.factory.initial".
 * 
 * </p>
 * <h2>Sample Configuration</h2> &lt;Config&gt;<br>
 * &nbsp;&lt;Locators&gt;<br>
 * &nbsp;&nbsp;&lt;Locator class="net.sf.corn.gate.locators.EJB2Locator"/&gt;<br>
 * &nbsp;&lt;/Locators&gt;<br>
 * &nbsp;&lt;Services&gt;<br>
 * &nbsp;&nbsp;&lt;Service name="echoBean" componentTarget="GWTTests-EJB2-EJB21Session"
 * scope="singleton" locator="ejb2"&gt;<br>
 * &nbsp;&nbsp;&nbsp;&lt;Operation pattern=".*" /&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;Property key="homeInterfaceClass"
 * value="com.dflora.gwt.sample.ejb2.EJB21SessionRemoteHome"/&gt;<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt!--INITIAL_CONTEXT_FACTORY--&gt<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;Property key="java.naming.factory.initial"
 * value="weblogic.jndi.WLInitialContextFactory" /&gt<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt!--PROVIDER_URL--&gt<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;Property key="java.naming.provider.url"
 * value="t3://localhost:7001" /&gt<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt!--SECURITY_PRINCIPAL--&gt<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;Property key="java.naming.security.principal"
 * value="weblogic" /&gt<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt!--SECURITY_CREDENTIALS--&gt<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&lt;Property key="java.naming.security.credentials"
 * value="weblogic1" /&gt<br>
 * &nbsp;&nbsp;&lt;/Service&gt;<br>
 * &nbsp;&lt;Services&gt;<br>
 * &lt;/Config&gt;<br>
 * 
 * @author <b>Serhat Dirik</b>
 * 
 */
public class EJB2Locator implements ILocator {

	private final static String NAME = "ejb2";
	private static Logger log = LoggerFactory.getLogger(EJB2Locator.class);
	private InitialContext ctx = null;
	public static final String HOME_INTERFACE_ATTRIBUTE = "homeInterfaceClass";
	public static final String HOME_CREATE_METHOD = "create";
	private static Set<String> contextAttributes = new CopyOnWriteArraySet<String>();
	private static ExceptionCommand cmdLog = new ExceptionCommand() {
		@Override
		public void execute(Exception e) {
			if (log.isErrorEnabled())
				log.error(e.getMessage(), e);

		}
	};
	static {
		Field[] fields = Context.class.getFields();
		for (Field field : fields) {
			try {
				if (field.getType().equals(String.class)) {
					String fieldVal = (String) field.get(null);
					contextAttributes.add(fieldVal);
				}
			} catch (Exception ex) {

			}
		}
	}

	public EJB2Locator() {
		try {
			ctx = new InitialContext();
		} catch (NamingException e) {
			throw new RuntimeException(e);
		}
	}

	private InitialContext prepContextForComponent(ServiceDefinition serviceDef)
			throws GateException {
		InitialContext ejbCtx = ctx;
		Hashtable<String, String> env = new Hashtable<String, String>();
		for (String contextAttributeName : contextAttributes) {
			String contextAttributeVal = serviceDef
					.getProperty(contextAttributeName);
			if (contextAttributeVal != null)
				env.put(contextAttributeName, contextAttributeVal);
		}
		// Try to initiate context with given parameters
		if (env.size() > 0) {
			try {
				ejbCtx = new InitialContext(env);
			} catch (Exception e) {
				throw ExceptionBuilder
						.with(GateException.class, e)
						.message(
								"An exception occured while initialing JNDI context for the component "
										+ serviceDef.getName()).execute(cmdLog)
						.build()
						.code(GateException.CODE_COMPONENT_INITIALIZATION);
			}
		}
		if (ejbCtx == null) {
			throw ExceptionBuilder.with(GateException.class)
					.message("Naming Context is not ready").execute(cmdLog)
					.build().code(GateException.CODE_COMPONENT_INITIALIZATION);
		}
		return ejbCtx;
	}

	private Class<?> findEjbHomeClass(ServiceDefinition serviceDef)
			throws GateException {
		String homeClassName = serviceDef.getProperty(HOME_INTERFACE_ATTRIBUTE);
		if (homeClassName == null || homeClassName.trim().length() == 0) {
			throw ExceptionBuilder
					.with(GateException.class)
					.message(
							"homeInterfaceClass attribute is empty in "
									+ serviceDef.getName()).execute(cmdLog)
					.build().code(GateException.CODE_COMPONENT_INITIALIZATION);
		}
		Class<?> hiClazz = null;
		try {
			hiClazz = Class.forName(homeClassName);
		} catch (ClassNotFoundException e) {
			throw ExceptionBuilder
					.with(GateException.class, e)
					.message(
							"homeInterfaceClass not found that defined in "
									+ serviceDef.getName()).execute(cmdLog)
					.build().code(GateException.CODE_COMPONENT_INITIALIZATION);
		}
		return hiClazz;
	}

	public Object getComponentInstance(ServiceDefinition serviceDef)
			throws GateException {

		InitialContext ejbCtx = prepContextForComponent(serviceDef);

		Class<?> hiClazz = findEjbHomeClass(serviceDef);

		try {
			Object homeObj = ejbCtx.lookup(serviceDef.getComponentTarget());

			Object nHomeObj = PortableRemoteObject.narrow(homeObj, hiClazz);
			Method createMethod = nHomeObj.getClass().getMethod(
					HOME_CREATE_METHOD);
			if (createMethod == null) {
				throw ExceptionBuilder
						.with(GateException.class)
						.message(
								"homeInterfaceClass doesn't contain create method in "
										+ serviceDef.getName()).execute(cmdLog)
						.build()
						.code(GateException.CODE_COMPONENT_INITIALIZATION);
			}
			Object component = createMethod.invoke(nHomeObj, new Object[] {});
			return component;
		} catch (NoSuchMethodException e) {
			throw ExceptionBuilder
					.with(GateException.class, e)
					.message(
							HOME_CREATE_METHOD
									+ " not defined in component definition of "
									+ serviceDef.getName()).execute(cmdLog)
					.build().code(GateException.CODE_COMPONENT_NOT_FOUND);
		} catch (Exception e) {
			throw ExceptionBuilder.with(GateException.class,e).message(e.getMessage()).execute(cmdLog).build().
					code(GateException.CODE_COMPONENT_INITIALIZATION);
		}

	}

	@Override
	public String getName() {
		return NAME;
	}

	@Override
	public String toString() {
		return getName() + " Locator";
	}

}
